<?php
namespace FM\Service;


use FM\Factory\GraficoFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class GraficoService {
    
    protected $connection;
    protected $GraficoFactory;
    protected $Environment;
    protected $newCode;
    
    public function __construct (EntityManager $entityManager, GraficoFactory $GraficoFactory, Environment $Environment, NewCode $newCode) {
        $this->connection = $entityManager->getConnection();
        $this->GraficoFactory  = $GraficoFactory;
        $this->Environment                  = $Environment;
        $this->newCode                      = $newCode;
    }
    
    public function getUnidade(){
        $dados = $this->GraficoFactory->getUnidade();
        return $dados;
    }
    
    public function getAtividades(){
        $dados = $this->GraficoFactory->getAtividades();
        return $dados;
    }
    
    public function getDadosGraficosGerados($param){
        $validParam = '';
        // debugApi([$param]);
        if ($param["informacaoBusca"] != '' && $param["informacaoBusca"] != null ) {
            $validParam = " WHERE (NMGRAFICOS) LIKE ('%".$param["informacaoBusca"]."%')";
        }
        
        $dados = $this->GraficoFactory->getDadosGraficosGerados($validParam);
        return $dados;
    }
    
    public function getFonteDados(){
        $dados = $this->GraficoFactory->getFonteDados();
        return $dados;
    }
    
    public function getMetricas($param){
        $parametro = '';
        
        if($param['selectObjectFonte'] != '' && $param['selectObjectFonte'] != null ) {
            $parametro = " WHERE (NRFONTEDADOSGRAFREL = ".$param['selectObjectFonte'].")";
        }
        
        $dados = $this->GraficoFactory->getMetricas($parametro);
        return $dados;
    }
    
    public function insertGrafico($param){
        $this->connection->beginTransaction();
        
        try {
            
            $nrgraficos = $this->newCode->getCodeByTableName("FCM_GRAFICOS", $this->Environment->getNrOrg(), 1, 10, true);
          
            $parametros = array(
                "NRGRAFICOS" => $nrgraficos,
                "NMGRAFICOS" => $param['NMGRAFICOS'],
                "NRFONTEDADOSGRAFREL" => $param['NRFONTEDADOSGRAFREL'],
                "IDTPGRAFICO" => $param['IDTPGRAFICO'],
                "IDPERIODO" => $param['IDPERIODO'],
                "DTINIGRAFICO" => $param['DTINIGRAFICO'],
                "DTFIMGRAFICO" => $param['DTFIMGRAFICO'],
                "IDGRAFICOPUB" => $param['IDGRAFICOPUB'],
                "TXCONSULTAGRAFICO" => $param['TXCONSULTAGRAFICO'],
                "NRORG" => $this->Environment->getNrOrg(),
                "IDATIVO" => $param['IDATIVO'],
                "CDOPERINCLUSAO" => $this->Environment->getCdOperador(),
                "NRORGINCLUSAO" => $this->Environment->getNrOrg()
            );
            
            $this->GraficoFactory->insertGrafico($parametros);
            
            forEach($param["METRICAS"] as $indices) {
                $nrconfgraficos = $this->newCode->getCodeByTableName("FCM_CONFGRAFICOS", $this->Environment->getNrOrg(), 1, 10, true);
                $parametro2 = array(
                    "NRCONFGRAFICO" => $nrconfgraficos,
                    "NRGRAFICOS" => $nrgraficos,
                    "IDDIMENSAODADO" => $indices["IDDIMENSAODADO"],
                    "NRDADOSGRAFREL" => $indices["NRDADOSGRAFREL"],
                    "NRORG" => $this->Environment->getNrOrg(),
                    "IDATIVO" => $param['IDATIVO'],
                    "CDOPERINCLUSAO" => $this->Environment->getCdOperador(),
                    "NRORGINCLUSAO" => $this->Environment->getNrOrg()
                );
                $this->GraficoFactory->insertConfGrafico($parametro2);
             
            }
            if ($param['NRFUNCIONARIO'] !== []) {
                
                forEach($param['NRFUNCIONARIO'] as $funcionarios) {
                    
                    $nrpermacessgrafico = $this->newCode->getCodeByTableName("FCM_PERMACESSGRAFICOS", $this->Environment->getNrOrg(), 1, 10, true);
                    
                    $parametros3 = array(
                        "NRPERMACESSGRAFICO" => $nrpermacessgrafico,
                        "NRGRAFICOS" => $nrgraficos,
                        "NRFUNCIONARIO" => $funcionarios,
                        "NRORG" => $this->Environment->getNrOrg(),
                        "IDATIVO" => $param['IDATIVO'],
                        "CDOPERINCLUSAO" => $this->Environment->getCdOperador(),
                        "NRORGINCLUSAO" => $this->Environment->getNrOrg()
                    );
                    
                    $this->GraficoFactory->insertPemissao($parametros3);
                }

            } 
            
            $this->connection->commit();
            $response = "OK";
        } 
        catch(\Exception $e){
            
            $this->connection->rollback();
            $response = $e->getMessage();
        }
        
        return $response;
    }
    
    public function deletaGraficoGerado($param){
        $this->connection->beginTransaction();
        try {
         $this->GraficoFactory->deletaConfGraficos($param["NRGRAFICOS"]);
         $this->GraficoFactory->deletaUsuariosPermitidos($param["NRGRAFICOS"]);
         $this->GraficoFactory->deletaGraficoGerado($param["NRGRAFICOS"]);
         $this->connection->commit();
         $response = "Deletado";
         
        } catch(\Exception $e) {
            $this->connection->rollback();
            $response = $e->getMessage();
        }
        return $response;
    }
    
    public function consultaDinamica($param) {
        $dados = $this->GraficoFactory->consultaDinamica($param);
        return $dados;
    }
    
    public function buscaDadosGrafRel($param) {
        $parametro = $param['parametro'];
        $dados = $this->GraficoFactory->buscaDadosGrafRel($parametro);
        return $dados;
    }
    
    public function getDadosEditGrafico($param) {
        $parametro = $param['params']['NRGRAFICOS'];
        $informacoesCampos = array();
        $dadosGrafico = $this->GraficoFactory->getDadosGraficos($parametro);
        $dadosConfgraficos = $this->GraficoFactory->getDadosConfGraficos($parametro);
        $dadosUsuarios = $this->GraficoFactory->getDadosUsuariosPermitidos($parametro);
        return [$dadosGrafico,$dadosConfgraficos,$dadosUsuarios];
    }
    
    public function copiaGrafico($param) {
        $parametro = $param['NRGRAFICOS'];
        $this->connection->beginTransaction();
        $NRORG = ' $this->Environment->getNrOrg()';
        try{
            
            $buscaGraficoGerado = $this->GraficoFactory->buscaDadosGrafico($parametro);
            
            $nrgraficos = $this->newCode->getCodeByTableName("FCM_GRAFICOS", $NRORG, 1, 10, true);
            
            forEach($buscaGraficoGerado as $resultado) {
                
                $retornoDadosGraficosGerados = array(
                    "NRGRAFICOS" => $nrgraficos,
                    "NMGRAFICOS" => $resultado['NMGRAFICOS'],
                    "NRFONTEDADOSGRAFREL" => $resultado['NRFONTEDADOSGRAFREL'],
                    "IDTPGRAFICO" => $resultado['IDTPGRAFICO'],
                    "IDPERIODO" => $resultado['IDPERIODO'],
                    "DTINIGRAFICO" => $resultado['DATAINICIO'],
                    "DTFIMGRAFICO" => $resultado['DATAFINAL'],
                    "IDGRAFICOPUB" => $resultado['IDGRAFICOPUB'],
                    "TXCONSULTAGRAFICO" => $resultado['TXCONSULTAGRAFICO'],
                    "NRORG" => $resultado['NRORG'],
                    "IDATIVO" => $resultado['IDATIVO'],
                    "CDOPERINCLUSAO" => $resultado['CDOPERINCLUSAO'],
                    "NRORGINCLUSAO" => $resultado['NRORGINCLUSAO']
                );
                
                
                $this->GraficoFactory->insertGrafico($retornoDadosGraficosGerados);
            }
            
            
            $buscaConfGraficos = $this->GraficoFactory->buscaDadosConfGrafico($parametro);
            
            forEach($buscaConfGraficos as $retornoConf) {
                
              $nrconfgraficos = $this->newCode->getCodeByTableName("FCM_CONFGRAFICOS",$NRORG, 1, 10, true);
                
                 $retornoDadosConfGraficos = array (
                    "NRCONFGRAFICO" => $nrconfgraficos,
                    "NRGRAFICOS" => $nrgraficos,
                    "IDDIMENSAODADO" => $retornoConf['IDDIMENSAODADO'],
                    "NRDADOSGRAFREL" => $retornoConf['NRDADOSGRAFREL'],
                    "NRORG" => $retornoConf['NRORG'],
                    "IDATIVO" => $retornoConf['IDATIVO'],
                    "CDOPERINCLUSAO" => $retornoConf['CDOPERINCLUSAO'],
                    "NRORGINCLUSAO" => $retornoConf['NRORGINCLUSAO']
                );
                
                 $this->GraficoFactory->insertConfGrafico($retornoDadosConfGraficos);
            }
            
            $buscaFuncionarios = $this->GraficoFactory->buscaDadosPermGrafico($parametro);
            
            if ($buscaFuncionarios !== []) {
                
                forEach($buscaFuncionarios as $retornoFuncionarios) {
                    
                    $nrpermacessgrafico = $this->newCode->getCodeByTableName("FCM_PERMACESSGRAFICOS",$NRORG, 1, 10, true);
                    
                    $retornoPermissao = array(
                        "NRPERMACESSGRAFICO" => $nrpermacessgrafico,
                        "NRGRAFICOS" => $nrgraficos,
                        "NRFUNCIONARIO" => $retornoFuncionarios['NRFUNCIONARIO'],
                        "NRORG" => $retornoFuncionarios['NRORG'],
                        "IDATIVO" => $retornoFuncionarios['IDATIVO'],
                        "CDOPERINCLUSAO" => $retornoFuncionarios['CDOPERINCLUSAO'],
                        "NRORGINCLUSAO" => $retornoFuncionarios['NRORGINCLUSAO'],
                    );
                    
                  $this->GraficoFactory->insertPemissao($retornoPermissao);
                }
                
            }
            
            $this->connection->commit();
            $response = "Copiado";
            
        } catch(\Exception $e) {
            
            $this->connection->rollback();
            $response = $e->getMessage();
        }
        
        return $response;
    }
    
    public function atualizaGrafico($param) {
        $nrgrafico = $param['NRGRAFICOS'];
        $this->connection->beginTransaction();
        $NRORG = $this->Environment->getNrOrg();
        try{
            
            $dadosGraficoUpdate =  array(
                "NMGRAFICOS" => $param['NMGRAFICOS'],
                "NRFONTEDADOSGRAFREL" => $param['NRFONTEDADOSGRAFREL'],
                "IDTPGRAFICO" => $param['IDTPGRAFICO'],
                "IDPERIODO" => $param['IDPERIODO'],
                "DTINIGRAFICO" => $param['DTINIGRAFICO'],
                "DTFIMGRAFICO" => $param['DTFIMGRAFICO'],
                "IDGRAFICOPUB" => $param['IDGRAFICOPUB'],
                "TXCONSULTAGRAFICO" => $param['TXCONSULTAGRAFICO'],
                "NRGRAFICOS" => $param['NRGRAFICOS'],
            );
            $this->GraficoFactory->updateGraficos($dadosGraficoUpdate);
            
            $verificaConfGrafico = $this->GraficoFactory->buscaDadosConfGrafico($nrgrafico);
            
            if ($verificaConfGrafico === []) {
                
                if ($param["METRICAS"] !== []) {
                    
                    forEach($param["METRICAS"] as $indices) {
                        
                        $nrconfgraficos = $this->newCode->getCodeByTableName("FCM_CONFGRAFICOS", $this->Environment->getNrOrg(), 1, 10, true);
                        
                        $parametro2 = array(
                            "NRCONFGRAFICO" => $nrconfgraficos,
                            "NRGRAFICOS" => $nrgrafico,
                            "IDDIMENSAODADO" => $indices["IDDIMENSAODADO"],
                            "NRDADOSGRAFREL" => $indices["NRDADOSGRAFREL"],
                            "NRORG" => $this->Environment->getNrOrg(),
                            "IDATIVO" => $param['IDATIVO'],
                            "CDOPERINCLUSAO" => $this->Environment->getCdOperador(),
                            "NRORGINCLUSAO" => $this->Environment->getNrOrg()
                        );
                        
                        $this->GraficoFactory->insertConfGrafico($parametro2);
                    }
                    
                }
            } else {
                
                $this->GraficoFactory->deletaConfGraficos($nrgrafico);
                
                if ($param["METRICAS"] !== []) {
                
                    forEach($param["METRICAS"] as $indices) {
                            
                        $nrconfgraficos = $this->newCode->getCodeByTableName("FCM_CONFGRAFICOS", $this->Environment->getNrOrg(), 1, 10, true);
                        
                        $parametro2 = array(
                            "NRCONFGRAFICO" => $nrconfgraficos,
                            "NRGRAFICOS" => $nrgrafico,
                            "IDDIMENSAODADO" => $indices["IDDIMENSAODADO"],
                            "NRDADOSGRAFREL" => $indices["NRDADOSGRAFREL"],
                            "NRORG" => $this->Environment->getNrOrg(),
                            "IDATIVO" => $param['IDATIVO'],
                            "CDOPERINCLUSAO" => $this->Environment->getCdOperador(),
                            "NRORGINCLUSAO" => $this->Environment->getNrOrg()
                        );
                        
                        $this->GraficoFactory->insertConfGrafico($parametro2);
                    }
                }
            }
            
            $verificaUsuariosPermitidos = $this->GraficoFactory->getDadosUsuariosPermitidos($nrgrafico);
            
            if ($verificaUsuariosPermitidos === []) {
                
                if ($param["NRFUNCIONARIO"] !== []) {
                    
                    forEach($param['NRFUNCIONARIO'] as $funcionarios) {
                    
                        $nrpermacessgrafico = $this->newCode->getCodeByTableName("FCM_PERMACESSGRAFICOS", $this->Environment->getNrOrg(), 1, 10, true);
                        
                        $parametros3 = array(
                            "NRPERMACESSGRAFICO" => $nrpermacessgrafico,
                            "NRGRAFICOS" => $nrgrafico,
                            "NRFUNCIONARIO" => $funcionarios,
                            "NRORG" => $this->Environment->getNrOrg(),
                            "IDATIVO" => $param['IDATIVO'],
                            "CDOPERINCLUSAO" => $this->Environment->getCdOperador(),
                            "NRORGINCLUSAO" => $this->Environment->getNrOrg()
                        );
                        
                        $this->GraficoFactory->insertPemissao($parametros3);
                    }
                }
            } else {
                
                $this->GraficoFactory->deletaUsuariosPermitidos($nrgrafico);
                
                if ($param["NRFUNCIONARIO"] !== []) {
                    
                    forEach($param['NRFUNCIONARIO'] as $funcionarios) {
                        
                        $nrpermacessgrafico = $this->newCode->getCodeByTableName("FCM_PERMACESSGRAFICOS", $this->Environment->getNrOrg(), 1, 10, true);
                        
                        $parametros3 = array(
                            "NRPERMACESSGRAFICO" => $nrpermacessgrafico,
                            "NRGRAFICOS" => $nrgrafico,
                            "NRFUNCIONARIO" => $funcionarios,
                            "NRORG" => $this->Environment->getNrOrg(),
                            "IDATIVO" => $param['IDATIVO'],
                            "CDOPERINCLUSAO" => $this->Environment->getCdOperador(),
                            "NRORGINCLUSAO" => $this->Environment->getNrOrg()
                        );
                        
                        $this->GraficoFactory->insertPemissao($parametros3);
                    }
                }
            }
            
            $this->connection->commit();
            $response = "Atualizado";
            
        } catch(\Exception $e) {
            
            $this->connection->rollback();
            $response = $e->getMessage();
        }
        
        return $response;
    }
    
    
}
?>