import {
  AlertService, Button, Date as DateInput, DialogService, Grid, Modal, ModalService, Select, SelectMultiple,
} from '@zeedhi/common';
import {
  dayjs, Http, IDictionary, Metadata,
} from '@zeedhi/core';

export class ModalTransfer {
  private PersistentModal!: Modal;

  public async createBasicUsageModal() {
    this.PersistentModal = await ModalService.createFromJson('/modal/modalTransfer', true);
  }

  public destroyBasicUsageModal() {
    this.PersistentModal.destroy();
  }

  public openBasicModal() {
    this.PersistentModal.show();
  }

  public closeBasicModal() {
    this.PersistentModal.hide();
  }

  public setEmployee({ component }:{ component:Select }) {
    component.datasource.data = [];
    Http.get('/Employee').then((response: IDictionary) => {
      component.datasource.data = response.data.dataset.employee;
    });
  }

  public setTask() {
    const funcionarioOrigem = Metadata.getInstance<Select>('SelectFuncionarioOrigem');
    const grid = Metadata.getInstance<Grid>('GridComTarefas');
    grid.datasource.data = [];
    if (funcionarioOrigem.value == null) {
      grid.isVisible = false;
    } else {
      const dataInicio = Metadata.getInstance<DateInput>('InicioPeridoTarefas').value || null;
      const dataFim = Metadata.getInstance<DateInput>('FimPeriodoTarefas').value || null;
      grid.isVisible = true;
      Http.get(`/taskOfTheEmployee?nr=${funcionarioOrigem.value}&inicio=${dataInicio}&fim=${dataFim}`).then((response: IDictionary) => {
        response.data.dataset.task.forEach((element: IDictionary) => {
          grid.datasource.data.push({
            id: element.NRCHECKLISTAGEND,
            name: element.NMCHECKLISTAGEND,
            local: element.NMLOCALCLIENTE,
            date: element.DTHORAAGEND,
          });
        });
      });
    }
  }

  public async transfer() {
    const grid = Metadata.getInstance<Grid>('GridComTarefas');
    const dadosHaAtualizar = grid.selectedRows.map((item: any) => item.id);
    let idVistoria = dadosHaAtualizar.filter((value: any) => value.indexOf('V-') >= 0);
    const idTarefa = dadosHaAtualizar.filter((value: any) => value.indexOf('V-') < 0);
    /* eslint-disable no-console */
    console.log(`Vistoria${idVistoria}\nTarefa: ${idTarefa}`);
    let funcionarioDestino = Metadata.getInstance<Select>('SelectFuncionarioDestino');
    const funcionarioOrigem = Metadata.getInstance<Select>('SelectFuncionarioOrigem').value;
    if (funcionarioDestino.isValid() && (idVistoria.length || idTarefa.length) && (funcionarioDestino.value !== funcionarioOrigem)) {
      funcionarioDestino = funcionarioDestino.value;
      let sucess = true;
      const operadorTransfer = localStorage.getItem('UserLogado');

      if (idVistoria.length) {
        idVistoria = idVistoria.map((value: string) => Number(value.substr(2)));
        await Http.post('/transferVistoria', {
          funcionarioDestino,
          idVistoria,
          operadorTransfer,
          funcionarioOrigem,
        }).then((response: IDictionary) => {
          sucess = response.data.dataset.status[0] === 'OK';
        });
      }

      if (idTarefa.length && sucess) {
        await Http.post('/transferTask', {
          funcionarioDestino,
          idTarefa,
          operadorTransfer,
          funcionarioOrigem,
        }).then((response: IDictionary) => {
          sucess = response.data.dataset.status[0] === 'OK';
        });
      }

      if (sucess) {
        grid.selectedRows = [];
        this.setTask();
        AlertService.show({
          name: 'basic-alert',
          text: 'Tarefas transferida com successo!',
          top: true,
          left: false,
          timeout: 3000,
          color: 'success',
        });
      } else {
        DialogService.show({
          name: 'basic-dialog',
          type: 'warning',
          title: 'Transação bloqueada!',
          text: 'Ocorreu um erro durante a transferencia.\nTente novamente mais tarde.\nSe o erro persistir, entre em contato com o administrador do sistema.',
          buttons: [
            {
              name: 'cancel',
              component: 'ZdButton',
              outline: true,
              label: 'Voltar',
              events: {
                click: () => DialogService.hide(),
              },
            },
          ],
        });
      }
    } else if (!funcionarioDestino.isValid()) {
      AlertService.show({
        name: 'basic-alert',
        text: 'Selecione um funcionário de destino.',
        top: true,
        left: false,
        timeout: 3000,
        color: '#e0ca36',
      });
    } else if (funcionarioDestino.value === funcionarioOrigem) {
      AlertService.show({
        name: 'basic-alert',
        text: 'Selecione um funcionário de destino diferente.',
        top: true,
        left: false,
        timeout: 3000,
        color: '#e0ca36',
      });
    } else {
      AlertService.show({
        name: 'basic-alert',
        text: 'Nenhuma Tarefa selecionada.',
        top: true,
        left: false,
        timeout: 3000,
        color: '#e0ca36',
      });
    }
  }

  private dadosOriginalDeTranferencia: any = [];

  public historicoTransfer({ component }:{ component:Grid }) {
    Http.get('/getHistorico').then((response: IDictionary) => {
      const dadosBanco = response.data.dataset.historico;
      let dadosHaModificar = dadosBanco.map((elemento: any) => elemento.NOME);
      Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_NOME').updateItems(dadosHaModificar.filter((elemento: any, pos: number) => dadosHaModificar.indexOf(elemento) === pos));
      dadosHaModificar = dadosBanco.map((elemento: any) => elemento.FUNCORIGEM);
      Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_FUNCORIGEM').updateItems(dadosHaModificar.filter((elemento: any, pos: number) => dadosHaModificar.indexOf(elemento) === pos));
      dadosHaModificar = dadosBanco.map((elemento: any) => elemento.FUNCDESTINO);
      Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_FUNCDESTINO').updateItems(dadosHaModificar.filter((elemento: any, pos: number) => dadosHaModificar.indexOf(elemento) === pos));
      dadosHaModificar = dadosBanco.map((elemento: any) => elemento.RESPONSAVEL);
      Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_RESPONSAVEL').updateItems(dadosHaModificar.filter((elemento: any, pos: number) => dadosHaModificar.indexOf(elemento) === pos));
      component.datasource.updateData(dadosBanco);
      this.dadosOriginalDeTranferencia = dadosBanco;
      component.datasource.get();
    });
  }

  public proximaPagina({ component }: any) {
    component.parent.datasource.page += 1;
    if (component.parent.paginationLength === component.parent.datasource.page) {
      component.disabled = true;
    }
    component.parent.datasource.get();
    Metadata.getInstance<Button>(`BotaoPaginaAnterior_${component.name.split('_')[1]}`).disabled = false;
    component.parent.datasource.get();
  }

  public paginaAnterior({ component }: any) {
    if (component.parent.datasource.page > 1) {
      component.parent.datasource.page -= 1;
      if (component.parent.datasource.page === 1) {
        component.disabled = true;
      }
      component.parent.datasource.get();
      Metadata.getInstance<Button>(`BotaoProximaPagina_${component.name.split('_')[1]}`).disabled = false;
    }
    component.parent.datasource.get();
  }

  public async filtrar() {
    const grid = Metadata.getInstance<Grid>('GridComTarefas');
    grid.datasource.updateData(this.dadosOriginalDeTranferencia);
    let copiaDatasource: any = grid.datasource;
    copiaDatasource = copiaDatasource.allData || grid.datasource.data;
    const dadosAFiltrarNome = Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_NOME').value;
    copiaDatasource = (dadosAFiltrarNome !== '' && dadosAFiltrarNome !== null && dadosAFiltrarNome.length > 0) ? copiaDatasource.filter((elemento: any) => dadosAFiltrarNome.includes(elemento.NOME)) : copiaDatasource;
    const dadosAFiltrarDataTarefa = Metadata.getInstance<DateInput>('CampoParaFiltrar_GridComTarefas_DTHORAAGEND').value;
    copiaDatasource = (dadosAFiltrarDataTarefa !== '' && dadosAFiltrarDataTarefa !== null && dadosAFiltrarDataTarefa.length === 10) ? copiaDatasource.filter((elemento: any) => dadosAFiltrarDataTarefa === elemento.DTHORAAGEND.split(' ')[0]) : copiaDatasource;
    const dadosAFiltrarFuncOrigem = Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_FUNCORIGEM').value;
    copiaDatasource = (dadosAFiltrarFuncOrigem !== '' && dadosAFiltrarFuncOrigem !== null && dadosAFiltrarFuncOrigem.length > 0) ? copiaDatasource.filter((elemento: any) => dadosAFiltrarFuncOrigem.includes(elemento.FUNCORIGEM)) : copiaDatasource;
    const dadosAFiltrarFuncDestino = Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_FUNCDESTINO').value;
    copiaDatasource = (dadosAFiltrarFuncDestino !== '' && dadosAFiltrarFuncDestino !== null && dadosAFiltrarFuncDestino.length > 0) ? copiaDatasource.filter((elemento: any) => dadosAFiltrarFuncDestino.includes(elemento.FUNCDESTINO)) : copiaDatasource;
    const dadosAFiltrarDataTransfer = Metadata.getInstance<DateInput>('CampoParaFiltrar_GridComTarefas_DTINCLUSAO').value;
    copiaDatasource = (dadosAFiltrarDataTransfer !== '' && dadosAFiltrarDataTransfer !== null && dadosAFiltrarDataTransfer.length === 10) ? copiaDatasource.filter((elemento: any) => dadosAFiltrarDataTransfer === elemento.DTINCLUSAO.split(' ')[0]) : copiaDatasource;
    const dadosAFiltrarResponsavel = Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_GridComTarefas_RESPONSAVEL').value;
    copiaDatasource = (dadosAFiltrarResponsavel !== '' && dadosAFiltrarResponsavel !== null && dadosAFiltrarResponsavel.length > 0) ? copiaDatasource.filter((elemento: any) => dadosAFiltrarResponsavel.includes(elemento.RESPONSAVEL)) : copiaDatasource;
    grid.datasource.updateData(copiaDatasource);
    grid.datasource.get();
  }

  public sortFunction(value1: any, value2: any): boolean {
    console.log(value1);
    return (dayjs(value1, 'DD/MM/YYYY H:m').isAfter(dayjs(value2, 'DD/MM/YYYY H:m')));
  }
}
